/*
 * Envers. http://www.jboss.org/envers
 *
 * Copyright 2008  Red Hat Middleware, LLC. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT A WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License, v.2.1 along with this distribution; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 *
 * Red Hat Author(s): Adam Warski
 */
package org.jboss.envers.query;

import org.jboss.envers.query.projection.VersionsProjection;
import org.jboss.envers.query.projection.DistinctVersionsProjection;
import org.jboss.envers.query.projection.ProjectionWrapper;
import org.jboss.envers.query.projection.VersionsProjectionList;
import org.hibernate.criterion.Projection;

/**
 * @author Adam Warski (adam at warski dot org)
 */
@SuppressWarnings({"JavaDoc"})
public class VersionsProjections {
    /**
     * A distinct projection on a given versions projection.
     * @param projection
     * @return
     */
    public static VersionsProjection distinct(VersionsProjection projection) {
        return new DistinctVersionsProjection(projection);
    }

    /**
     * A distinct projection on a given hibernate projection, which can be later used together with
     * other versions projections (for example in a list). 
     */
    public static VersionsProjection distinct(Projection projection) {
        return new DistinctVersionsProjection(new ProjectionWrapper(projection));
    }

    /**
     * A list of versions and hibernate projections.
     */
    public static VersionsProjectionList projectionList() {
        return new VersionsProjectionList();
    }
}
